@@ -1,5 +1,6 @@
 from datetime import date, datetime
 
+import six
 from django.conf import settings
 from django.core.mail.backends.base import BaseEmailBackend
 from django.utils.timezone import is_naive, get_current_timezone, make_aware, utc
@@ -7,8 +8,8 @@ from django.utils.timezone import is_naive, get_current_timezone, make_aware, ut
 from ..exceptions import AnymailCancelSend, AnymailError, AnymailUnsupportedFeature, AnymailRecipientsRefused
 from ..message import AnymailStatus
 from ..signals import pre_send, post_send
-from ..utils import (Attachment, ParsedEmail, UNSET, combine, last, get_anymail_setting,
-                     force_non_lazy, force_non_lazy_list, force_non_lazy_dict)
+from ..utils import (Attachment, UNSET, combine, last, get_anymail_setting, parse_address_list,
+                     force_non_lazy, force_non_lazy_list, force_non_lazy_dict, is_lazy)
 
 
 class AnymailBaseBackend(BaseEmailBackend):
@@ -215,12 +216,12 @@ class BasePayload(object):
     # the combined/converted results for each attr.
     base_message_attrs = (
         # Standard EmailMessage/EmailMultiAlternatives props
-        ('from_email', last, 'parsed_email'),
-        ('to', combine, 'parsed_emails'),
-        ('cc', combine, 'parsed_emails'),
-        ('bcc', combine, 'parsed_emails'),
+        ('from_email', last, parse_address_list),  # multiple from_emails are allowed
+        ('to', combine, parse_address_list),
+        ('cc', combine, parse_address_list),
+        ('bcc', combine, parse_address_list),
         ('subject', last, force_non_lazy),
-        ('reply_to', combine, 'parsed_emails'),
+        ('reply_to', combine, parse_address_list),
         ('extra_headers', combine, force_non_lazy_dict),
         ('body', last, force_non_lazy),  # special handling below checks message.content_subtype
         ('alternatives', combine, 'prepped_alternatives'),
@@ -252,6 +253,8 @@ class BasePayload(object):
         message_attrs = self.base_message_attrs + self.anymail_message_attrs + self.esp_message_attrs
         for attr, combiner, converter in message_attrs:
             value = getattr(message, attr, UNSET)
+            if attr in ('to', 'cc', 'bcc', 'reply_to') and value is not UNSET:
+                self.validate_not_bare_string(attr, value)
             if combiner is not None:
                 default_value = self.defaults.get(attr, UNSET)
                 value = combiner(default_value, value)
@@ -263,6 +266,8 @@ class BasePayload(object):
             if value is not UNSET:
                 if attr == 'body':
                     setter = self.set_html_body if message.content_subtype == 'html' else self.set_text_body
+                elif attr == 'from_email':
+                    setter = self.set_from_email_list
                 else:
                     # AttributeError here? Your Payload subclass is missing a set_<attr> implementation
                     setter = getattr(self, 'set_%s' % attr)
@@ -274,16 +279,28 @@ class BasePayload(object):
                                             email_message=self.message, payload=self, backend=self.backend)
 
     #
-    # Attribute converters
+    # Attribute validators
     #
 
-    def parsed_email(self, address):
-        return ParsedEmail(address, self.message.encoding)  # (handles lazy address)
+    def validate_not_bare_string(self, attr, value):
+        """EmailMessage to, cc, bcc, and reply_to are specced to be lists of strings.
 
-    def parsed_emails(self, addresses):
-        encoding = self.message.encoding
-        return [ParsedEmail(address, encoding)  # (handles lazy address)
-                for address in addresses]
+        This catches the common error where a single string is used instead.
+        (See also checks in EmailMessage.__init__.)
+        """
+        # Note: this actually only runs for reply_to. If to, cc, or bcc are
+        # set to single strings, you'll end up with an earlier cryptic TypeError
+        # from EmailMesssage.recipients (called from EmailMessage.send) before
+        # the Anymail backend even gets involved:
+        #   TypeError: must be str, not list
+        #   TypeError: can only concatenate list (not "str") to list
+        #   TypeError: Can't convert 'list' object to str implicitly
+        if isinstance(value, six.string_types) or is_lazy(value):
+            raise TypeError('"{attr}" attribute must be a list or other iterable'.format(attr=attr))
+
+    #
+    # Attribute converters
+    #
 
     def prepped_alternatives(self, alternatives):
         return [(force_non_lazy(content), mimetype)
@@ -325,8 +342,17 @@ class BasePayload(object):
         raise NotImplementedError("%s.%s must implement init_payload" %
                                   (self.__class__.__module__, self.__class__.__name__))
 
+    def set_from_email_list(self, emails):
+        # If your backend supports multiple from emails, override this to handle the whole list;
+        # otherwise just implement set_from_email
+        if len(emails) > 1:
+            self.unsupported_feature("multiple from emails")
+            # fall through if ignoring unsupported features
+        if len(emails) > 0:
+            self.set_from_email(emails[0])
+
     def set_from_email(self, email):
-        raise NotImplementedError("%s.%s must implement set_from_email" %
+        raise NotImplementedError("%s.%s must implement set_from_email or set_from_email_list" %
                                   (self.__class__.__module__, self.__class__.__name__))
 
     def set_to(self, emails):
