@@ -1,7 +1,6 @@
-import warnings
 from datetime import datetime
 
-from ..exceptions import AnymailRequestsAPIError, AnymailError, AnymailDeprecationWarning
+from ..exceptions import AnymailRequestsAPIError, AnymailError
 from ..message import AnymailRecipientStatus
 from ..utils import get_anymail_setting, rfc2822date
 
@@ -54,16 +53,7 @@ class EmailBackend(AnymailRequestsBackend):
                                           backend=self)
         # Simulate a per-recipient status of "queued":
         status = AnymailRecipientStatus(message_id=message_id, status="queued")
-        return {recipient.email: status for recipient in payload.all_recipients}
-
-
-# Pre-v0.8 naming (deprecated)
-class MailgunBackend(EmailBackend):
-    def __init__(self, **kwargs):
-        warnings.warn(AnymailDeprecationWarning(
-            "Please update your EMAIL_BACKEND setting to "
-            "'anymail.backends.mailgun.EmailBackend'"))
-        super(MailgunBackend, self).__init__(**kwargs)
+        return {recipient.addr_spec: status for recipient in payload.all_recipients}
 
 
 class MailgunPayload(RequestsPayload):
@@ -137,7 +127,7 @@ class MailgunPayload(RequestsPayload):
             self.data[recipient_type] = [email.address for email in emails]
             self.all_recipients += emails  # used for backend.parse_recipient_status
         if recipient_type == 'to':
-            self.to_emails = [email.email for email in emails]  # used for populate_recipient_variables
+            self.to_emails = [email.addr_spec for email in emails]  # used for populate_recipient_variables
 
     def set_subject(self, subject):
         self.data["subject"] = subject
