@@ -1,9 +1,8 @@
 import re
-import warnings
 
 from requests.structures import CaseInsensitiveDict
 
-from ..exceptions import AnymailRequestsAPIError, AnymailDeprecationWarning
+from ..exceptions import AnymailRequestsAPIError
 from ..message import AnymailRecipientStatus
 from ..utils import get_anymail_setting
 
@@ -70,9 +69,9 @@ class EmailBackend(AnymailRequestsBackend):
                                           backend=self)
 
         return {
-            recipient.email: AnymailRecipientStatus(
+            recipient.addr_spec: AnymailRecipientStatus(
                 message_id=message_id,
-                status=('rejected' if recipient.email.lower() in rejected_emails
+                status=('rejected' if recipient.addr_spec.lower() in rejected_emails
                         else default_status)
             )
             for recipient in payload.all_recipients
@@ -95,15 +94,6 @@ class EmailBackend(AnymailRequestsBackend):
             return []
 
 
-# Pre-v0.8 naming (deprecated)
-class PostmarkBackend(EmailBackend):
-    def __init__(self, **kwargs):
-        warnings.warn(AnymailDeprecationWarning(
-            "Please update your EMAIL_BACKEND setting to "
-            "'anymail.backends.postmark.EmailBackend'"))
-        super(PostmarkBackend, self).__init__(**kwargs)
-
-
 class PostmarkPayload(RequestsPayload):
 
     def __init__(self, message, defaults, backend, *args, **kwargs):
