@@ -5,9 +5,9 @@ from django.core.mail import make_msgid
 from requests.structures import CaseInsensitiveDict
 
 from .base_requests import AnymailRequestsBackend, RequestsPayload
-from ..exceptions import AnymailConfigurationError, AnymailRequestsAPIError, AnymailWarning, AnymailDeprecationWarning
+from ..exceptions import AnymailConfigurationError, AnymailRequestsAPIError, AnymailWarning
 from ..message import AnymailRecipientStatus
-from ..utils import get_anymail_setting, timestamp, update_deep
+from ..utils import BASIC_NUMERIC_TYPES, get_anymail_setting, timestamp, update_deep, parse_address_list
 
 
 class EmailBackend(AnymailRequestsBackend):
@@ -64,16 +64,7 @@ class EmailBackend(AnymailRequestsBackend):
         # SendGrid v3 doesn't provide any information in the response for a successful send,
         # so simulate a per-recipient status of "queued":
         status = AnymailRecipientStatus(message_id=payload.message_id, status="queued")
-        return {recipient.email: status for recipient in payload.all_recipients}
-
-
-# Pre-v0.8 naming (deprecated)
-class SendGridBackend(EmailBackend):
-    def __init__(self, **kwargs):
-        warnings.warn(AnymailDeprecationWarning(
-            "Please update your EMAIL_BACKEND setting to "
-            "'anymail.backends.sendgrid.EmailBackend'"))
-        super(SendGridBackend, self).__init__(**kwargs)
+        return {recipient.addr_spec: status for recipient in payload.all_recipients}
 
 
 class SendGridPayload(RequestsPayload):
@@ -115,7 +106,7 @@ class SendGridPayload(RequestsPayload):
         if "Reply-To" in headers:
             # Reply-To must be in its own param
             reply_to = headers.pop('Reply-To')
-            self.set_reply_to([self.parsed_email(reply_to)])
+            self.set_reply_to(parse_address_list([reply_to]))
         if len(headers) > 0:
             self.data["headers"] = dict(headers)  # flatten to normal dict for json serialization
         else:
@@ -208,17 +199,17 @@ class SendGridPayload(RequestsPayload):
 
     @staticmethod
     def email_object(email, workaround_name_quote_bug=False):
-        """Converts ParsedEmail to SendGrid API {email, name} dict"""
-        obj = {"email": email.email}
-        if email.name:
+        """Converts EmailAddress to SendGrid API {email, name} dict"""
+        obj = {"email": email.addr_spec}
+        if email.display_name:
             # Work around SendGrid API bug: v3 fails to properly quote display-names
             # containing commas or semicolons in personalizations (but not in from_email
             # or reply_to). See https://github.com/sendgrid/sendgrid-python/issues/291.
             # We can work around the problem by quoting the name for SendGrid.
             if workaround_name_quote_bug:
-                obj["name"] = '"%s"' % rfc822_quote(email.name)
+                obj["name"] = '"%s"' % rfc822_quote(email.display_name)
             else:
-                obj["name"] = email.name
+                obj["name"] = email.display_name
         return obj
 
     def set_from_email(self, email):
@@ -249,7 +240,7 @@ class SendGridPayload(RequestsPayload):
         # SendGrid requires header values to be strings -- not integers.
         # We'll stringify ints and floats; anything else is the caller's responsibility.
         self.data["headers"].update({
-            k: str(v) if isinstance(v, (int, float)) else v
+            k: str(v) if isinstance(v, BASIC_NUMERIC_TYPES) else v
             for k, v in headers.items()
         })
 
@@ -299,7 +290,7 @@ class SendGridPayload(RequestsPayload):
         # if they're not.)
         # We'll stringify ints and floats; anything else is the caller's responsibility.
         self.data["custom_args"] = {
-            k: str(v) if isinstance(v, (int, float)) else v
+            k: str(v) if isinstance(v, BASIC_NUMERIC_TYPES) else v
             for k, v in metadata.items()
         }
 
@@ -344,4 +335,3 @@ class SendGridPayload(RequestsPayload):
                 "or use 'anymail.backends.sendgrid_v2.EmailBackend' for the old API."
             )
         update_deep(self.data, extra)
-
