@@ -1,10 +1,7 @@
 from __future__ import absolute_import  # we want the sparkpost package, not our own module
 
-import warnings
-
 from .base import AnymailBaseBackend, BasePayload
-from ..exceptions import (AnymailAPIError, AnymailImproperlyInstalled,
-                          AnymailConfigurationError, AnymailDeprecationWarning)
+from ..exceptions import AnymailAPIError, AnymailImproperlyInstalled, AnymailConfigurationError
 from ..message import AnymailRecipientStatus
 from ..utils import get_anymail_setting
 
@@ -79,16 +76,7 @@ class EmailBackend(AnymailBaseBackend):
         else:  # mixed results, or wrong total
             status = 'unknown'
         recipient_status = AnymailRecipientStatus(message_id=transmission_id, status=status)
-        return {recipient.email: recipient_status for recipient in payload.all_recipients}
-
-
-# Pre-v0.8 naming (deprecated)
-class SparkPostBackend(EmailBackend):
-    def __init__(self, **kwargs):
-        warnings.warn(AnymailDeprecationWarning(
-            "Please update your EMAIL_BACKEND setting to "
-            "'anymail.backends.sparkpost.EmailBackend'"))
-        super(SparkPostBackend, self).__init__(**kwargs)
+        return {recipient.addr_spec: recipient_status for recipient in payload.all_recipients}
 
 
 class SparkPostPayload(BasePayload):
@@ -104,11 +92,11 @@ class SparkPostPayload(BasePayload):
         if len(self.merge_data) > 0:
             # Build JSON recipient structures
             for email in self.to_emails:
-                rcpt = {'address': {'email': email.email}}
-                if email.name:
-                    rcpt['address']['name'] = email.name
+                rcpt = {'address': {'email': email.addr_spec}}
+                if email.display_name:
+                    rcpt['address']['name'] = email.display_name
                 try:
-                    rcpt['substitution_data'] = self.merge_data[email.email]
+                    rcpt['substitution_data'] = self.merge_data[email.addr_spec]
                 except KeyError:
                     pass  # no merge_data or none for this recipient
                 recipients.append(rcpt)
@@ -129,8 +117,10 @@ class SparkPostPayload(BasePayload):
 
         return self.params
 
-    def set_from_email(self, email):
-        self.params['from_email'] = email.address
+    def set_from_email_list(self, emails):
+        # SparkPost supports multiple From email addresses,
+        # as a single comma-separated string
+        self.params['from_email'] = ", ".join([email.address for email in emails])
 
     def set_to(self, emails):
         if emails:
