@@ -12,12 +12,16 @@ from django.conf import settings
 from django.core.mail.message import sanitize_address, DEFAULT_ATTACHMENT_MIME_TYPE
 from django.utils.encoding import force_text
 from django.utils.functional import Promise
-from django.utils.timezone import utc
+from django.utils.timezone import utc, get_fixed_timezone
 # noinspection PyUnresolvedReferences
 from six.moves.urllib.parse import urlsplit, urlunsplit
 
 from .exceptions import AnymailConfigurationError, AnymailInvalidAddress
 
+
+BASIC_NUMERIC_TYPES = six.integer_types + (float,)  # int, float, and (on Python 2) long
+
+
 UNSET = object()  # Used as non-None default value
 
 
@@ -114,7 +118,7 @@ def update_deep(dct, other):
 
 
 def parse_address_list(address_list):
-    """Returns a list of ParsedEmail objects from strings in address_list.
+    """Returns a list of EmailAddress objects from strings in address_list.
 
     Essentially wraps :func:`email.utils.getaddresses` with better error
     messaging and more-useful output objects
@@ -124,7 +128,7 @@ def parse_address_list(address_list):
 
     :param list[str]|str|None|list[None] address_list:
         the address or addresses to parse
-    :return list[:class:`ParsedEmail`]:
+    :return list[:class:`EmailAddress`]:
     :raises :exc:`AnymailInvalidAddress`:
     """
     if isinstance(address_list, six.string_types) or is_lazy(address_list):
@@ -141,14 +145,15 @@ def parse_address_list(address_list):
     name_email_pairs = getaddresses(address_list_strings)
     if name_email_pairs == [] and address_list_strings == [""]:
         name_email_pairs = [('', '')]  # getaddresses ignores a single empty string
-    parsed = [ParsedEmail(name_email_pair) for name_email_pair in name_email_pairs]
+    parsed = [EmailAddress(display_name=name, addr_spec=email)
+              for (name, email) in name_email_pairs]
 
     # Sanity-check, and raise useful errors
     for address in parsed:
-        if address.localpart == '' or address.domain == '':
-            # Django SMTP allows localpart-only emails, but they're not meaningful with an ESP
+        if address.username == '' or address.domain == '':
+            # Django SMTP allows username-only emails, but they're not meaningful with an ESP
             errmsg = "Invalid email address '%s' parsed from '%s'." % (
-                address.email, ", ".join(address_list_strings))
+                address.addr_spec, ", ".join(address_list_strings))
             if len(parsed) > len(address_list):
                 errmsg += " (Maybe missing quotes around a display-name?)"
             raise AnymailInvalidAddress(errmsg)
@@ -156,46 +161,46 @@ def parse_address_list(address_list):
     return parsed
 
 
-class ParsedEmail(object):
-    """A sanitized, complete email address with separate name and email properties.
+class EmailAddress(object):
+    """A sanitized, complete email address with easy access
+    to display-name, addr-spec (email), etc.
 
-    (Intended for Anymail internal use.)
+    Similar to Python 3.6+ email.headerregistry.Address
 
     Instance properties, all read-only:
-    :ivar str name:
+    :ivar str display_name:
         the address's display-name portion (unqouted, unescaped),
         e.g., 'Display Name, Inc.'
-    :ivar str email:
+    :ivar str addr_spec:
         the address's addr-spec portion (unquoted, unescaped),
         e.g., 'user@example.com'
+    :ivar str username:
+        the local part (before the '@') of the addr-spec,
+        e.g., 'user'
+    :ivar str domain:
+        the domain part (after the '@') of the addr-spec,
+        e.g., 'example.com'
+
     :ivar str address:
         the fully-formatted address, with any necessary quoting and escaping,
         e.g., '"Display Name, Inc." <user@example.com>'
-    :ivar str localpart:
-        the local part (before the '@') of email,
-        e.g., 'user'
-    :ivar str domain:
-        the domain part (after the '@') of email,
-        e.g., 'example.com'
+        (also available as `str(EmailAddress)`)
     """
 
-    def __init__(self, name_email_pair):
-        """Construct a ParsedEmail.
-
-        You generally should use :func:`parse_address_list` rather than creating
-        ParsedEmail objects directly.
-
-        :param tuple(str, str) name_email_pair:
-            the display-name and addr-spec (both unquoted) for the address,
-            as returned by :func:`email.utils.parseaddr` and
-            :func:`email.utils.getaddresses`
-        """
+    def __init__(self, display_name='', addr_spec=None):
         self._address = None  # lazy formatted address
-        self.name, self.email = name_email_pair
+        if addr_spec is None:
+            try:
+                display_name, addr_spec = display_name  # unpack (name,addr) tuple
+            except ValueError:
+                pass
+        self.display_name = display_name
+        self.addr_spec = addr_spec
         try:
-            self.localpart, self.domain = self.email.split("@", 1)
+            self.username, self.domain = addr_spec.split("@", 1)
+            # do we need to unquote username?
         except ValueError:
-            self.localpart = self.email
+            self.username = addr_spec
             self.domain = ''
 
     @property
@@ -211,7 +216,7 @@ class ParsedEmail(object):
         """Return a fully-formatted email address, using encoding.
 
         This is essentially the same as :func:`email.utils.formataddr`
-        on the ParsedEmail's name and email properties, but uses
+        on the EmailAddress's name and email properties, but uses
         Django's :func:`~django.core.mail.message.sanitize_address`
         for improved PY2/3 compatibility, consistent handling of
         encoding (a.k.a. charset), and proper handling of IDN
@@ -222,7 +227,7 @@ class ParsedEmail(object):
             default None uses ascii if possible, else 'utf-8'
             (quoted-printable utf-8/base64)
         """
-        return sanitize_address((self.name, self.email), encoding)
+        return sanitize_address((self.display_name, self.addr_spec), encoding)
 
     def __str__(self):
         return self.address
@@ -377,6 +382,34 @@ def collect_all_methods(cls, method_name):
     return methods
 
 
+def querydict_getfirst(qdict, field, default=UNSET):
+    """Like :func:`django.http.QueryDict.get`, but returns *first* value of multi-valued field.
+
+    >>> from django.http import QueryDict
+    >>> q = QueryDict('a=1&a=2&a=3')
+    >>> querydict_getfirst(q, 'a')
+    '1'
+    >>> q.get('a')
+    '3'
+    >>> q['a']
+    '3'
+
+    You can bind this to a QueryDict instance using the "descriptor protocol":
+    >>> q.getfirst = querydict_getfirst.__get__(q)
+    >>> q.getfirst('a')
+    '1'
+    """
+    # (Why not instead define a QueryDict subclass with this method? Because there's no simple way
+    # to efficiently initialize a QueryDict subclass with the contents of an existing instance.)
+    values = qdict.getlist(field)
+    if len(values) > 0:
+        return values[0]
+    elif default is not UNSET:
+        return default
+    else:
+        return qdict[field]  # raise appropriate KeyError
+
+
 EPOCH = datetime(1970, 1, 1, tzinfo=utc)
 
 
@@ -400,6 +433,18 @@ def rfc2822date(dt):
     return formatdate(timeval, usegmt=True)
 
 
+def angle_wrap(s):
+    """Return s surrounded by angle brackets, added only if necessary"""
+    # This is the inverse behavior of email.utils.unquote
+    # (which you might think email.utils.quote would do, but it doesn't)
+    if len(s) > 0:
+        if s[0] != '<':
+            s = '<' + s
+        if s[-1] != '>':
+            s = s + '>'
+    return s
+
+
 def is_lazy(obj):
     """Return True if obj is a Django lazy object."""
     # See django.utils.functional.lazy. (This appears to be preferred
@@ -462,3 +507,40 @@ def get_request_uri(request):
         url = urlunsplit((parts.scheme, basic_auth + '@' + parts.netloc,
                           parts.path, parts.query, parts.fragment))
     return url
+
+
+try:
+    from email.utils import parsedate_to_datetime  # Python 3.3+
+except ImportError:
+    from email.utils import parsedate_tz
+
+    # Backport Python 3.3+ email.utils.parsedate_to_datetime
+    def parsedate_to_datetime(s):
+        # *dtuple, tz = _parsedate_tz(data)
+        dtuple = parsedate_tz(s)
+        tz = dtuple[-1]
+        # if tz is None:  # parsedate_tz returns 0 for "-0000"
+        if tz is None or (tz == 0 and "-0000" in s):
+            # "... indicates that the date-time contains no information
+            # about the local time zone" (RFC 2822 #3.3)
+            return datetime(*dtuple[:6])
+        else:
+            # tzinfo = datetime.timezone(datetime.timedelta(seconds=tz))  # Python 3.2+ only
+            tzinfo = get_fixed_timezone(tz // 60)  # don't use timedelta (avoid Django bug #28739)
+            return datetime(*dtuple[:6], tzinfo=tzinfo)
+
+
+def parse_rfc2822date(s):
+    """Parses an RFC-2822 formatted date string into a datetime.datetime
+
+    Returns None if string isn't parseable. Returned datetime will be naive
+    if string doesn't include known timezone offset; aware if it does.
+
+    (Same as Python 3 email.utils.parsedate_to_datetime, with improved
+    handling for unparseable date strings.)
+    """
+    try:
+        return parsedate_to_datetime(s)
+    except (IndexError, TypeError, ValueError):
+        # despite the docs, parsedate_to_datetime often dies on unparseable input
+        return None
